﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

/*
 * O rekurencji myślimy w sytuacji, kiedy mają Państwo do czynienia z programem, w którym
 * z jakiegoś powodu (obliczeniowego lub łatwości implementacji) dobrze jest w danej funkcji
 * odwołać się raz jeszcze do tej samej funkcji.
 * To tak, jakby w funkcji Suma(a, b) znalazło się odwołanie Suma(a, b).
 * 
 * W praktyce rekurencja wielokrotnie ułatwi Państwa pracę, np. w przypadku implementacji drzew.
 * Obecnie najprostszym, ale też jednocześnie najczęstszym przykładem jest implementacja funkcji, 
 * która oblicza n-ty wyraz ciągu Fibonacciego lub silni.
 * 
 * Poniżej znajduje się przykładowe - rekurencyjne rozwiązanie ciągu Fibonacciego.
 */

namespace Rekurencja
{
    class Program
    {
        /*
         * Proszę zwrócić uwagę na stronę:
         * https://pl.wikipedia.org/wiki/Ci%C4%85g_Fibonacciego
         * Tak na prawdę podany wzór można bezpośrednio przełożyć na implementację rekurencyjnej
         * wersji ciągu Fibonacciego. Poniższa implementacja nie jest maksymalnie skrócona, ale jest
         * za to możliwie czytelna.
         */
        static long CiągFibonacciego(long n)
        {
            long fib;
            if (n == 0)
            {
                fib = 0;
            }
            else if (n == 1)
            {
                fib = 1;
            }
            else
            {
                fib = CiągFibonacciego(n - 1) + CiągFibonacciego(n - 2); // W tej chwili będąc w funkcji CiągFibonacciego(n) wywołujemy ją ponownie.
            }
            return fib;
        }

        static void Main(string[] args)
        {
            /*
             * Przykładowe wywołanie funkcji - funkcja jest wywołana 10 razy, dla pierwszych 10-ciu wyrazów ciągu.
             */
            for (int i = 0; i < 11; i++) { 
                long fib = CiągFibonacciego(i);
                Console.WriteLine(fib);
            }
        }
    }
}
